@@ -12,7 +12,7 @@ from sklearn.preprocessing import StandardScaler
 from sklearn.model_selection import GridSearchCV, train_test_split
 from sklearn.metrics import SCORERS
 from sklearn.exceptions import UndefinedMetricWarning
-from sklearn.feature_selection import SelectKBest, chi2
+from sklearn.feature_selection import SelectKBest, chi2, VarianceThreshold, RFE, SelectFromModel, SequentialFeatureSelector
 from difflib import get_close_matches
 from copy import deepcopy
 import numpy as np
@@ -33,16 +33,20 @@ class Model():
 
     Attributes
     ----------
+    :X : np.ndarray
+        training data.
+    :Y : np.ndarray
+        training data labels.
     :algorithm : str
         sklearn regression algorithm to build and fit model with. Value can be 
         an approximate representation of model name, for example: 'plsreg' will 
-        initialiase an instance of the PLSRegression model. Available algorithms
-        listed above.
+        initialiase an instance of the PLSRegression model etc. Available 
+        algorithms listed above.
     :parameters : dict (default={})
-        parameters to use for building regression model, by default it is set to {}, 
-        meaning all of the models' default parameters are used. Refer to sci-kit
-        learn for full list of available input parameters for each model: 
-        https://scikit-learn.org/stable/index.html.
+        parameters to use for specific sklearn model when building regression 
+        model, by default it is set to {}, meaning all of the models' default 
+        parameters are used. Refer to sci-kit learn for full list of available 
+        input parameters for each model: https://scikit-learn.org/stable/index.html.
     :test_split : float (default=0.2)
         proportion of the test data to use for building model, default of 0.2 is 
         reccomended.
@@ -51,34 +55,45 @@ class Model():
     -------
     get_model():
         build model using inputtted parameters.
-    train_test_split(X, Y, scale = True, test_size = 0.2, random_state=None, shuffle=True):
+    train_test_split(scale = True, test_split = 0.2, random_state=None, shuffle=True):
         get train-test split of dataset.
     fit():
         fit model.
     predict():
-        predict activity values using model and test data.
+        predict activity values using trained model and test data.
     save(save_folder):
         save fitted model to save_folder.
-    hyperparameter_tuning(parameters, metric='r2', cv=5):
-        complete hyperparameter tuning of model and its associated parameters.
     model_fitted():
         return if model has been fitted (true or false)
+    hyperparameter_tuning(self, parameters={}, metric='r2', cv=5, n_jobs=None, verbose=2):
+        complete hyperparameter tuning of model and its associated parameters.
+    feature_selection(method=""):
+        undertake feature selection using technique specified by method input
+        parameter to find optimal selection of features for maximum predictability
+        in model.
     """
-    def __init__(self, algorithm, parameters={}, test_split=0.2):
+    def __init__(self, X, Y, algorithm, parameters={}, test_split=0.2):
 
         self.algorithm = algorithm
         self.test_split = test_split
+        self.X = X
+        self.Y = Y
 
-        #if no parameters input, then set to {} meaning default parameters are used
+        #if no model parameters input, then set to {} meaning default models' parameters are used
         if (parameters == [] or parameters == ""):
             self.parameters = {}
         else:
             self.parameters = parameters
 
         #list of valid models available to use for this class
-        self.valid_models = ['plsregression','randomforestregressor','adaboostregressor',\
-                            'baggingregressor','decisiontreeregressor', 'gbr', 'gradientboostingregressor', 
-                            'linearregression', 'lasso', 'ridge', 'svr', 'sgd', 'kneighborsregressor', 'knn']
+        self.valid_models = ['plsregression', 'randomforestregressor', 'adaboostregressor',\
+                            'baggingregressor', 'decisiontreeregressor', 'gbr', 'gradientboostingregressor', 
+                            'linearregression', 'lasso', 'ridge', 'svr', 'supportvectorregression', 'sgd',
+                            'stochasticgradientdescent', 'kneighborsregressor', 'knearestneighbors', 'knn']
+
+        #raise error if algorithm parameter isnt string type
+        if not(isinstance(self.algorithm, str)):
+            raise TypeError("Algorithm input parameter must be a string.")
 
         #get closest match of valid model from the input algorithm parameter value using difflib
         model_matches = get_close_matches(self.algorithm.lower().strip(),[item.lower().strip() \
@@ -91,8 +106,10 @@ class Model():
             raise ValueError('Input algorithm {} not found in list of available valid models {}.'.format(
                     self.algorithm, self.valid_models))
 
-        #create instance of algorithm object
+        #create instance of algorithm object using its sklearn constructor
         self.model = self.get_model()
+
+        #set model_fit to None, specifies if model has been fit or not
         self.model_fit = None
 
     def get_model(self):
@@ -110,15 +127,16 @@ class Model():
         :model : sklearn.model
             instantiated regression model with default or user-specified parameters.
         """
+        #use if/elif statements to get matching model specified by user in algorithm attribute
         if (self.algorithm.lower().strip() == 'plsregression'):
 
             #get parameters of sklearn model and check that user inputted
-            #  parameters are available in the model, only use those that are valid.
+            #parameters are available in the model, only use those that are valid.
             model_params = set(dir(PLSRegression()))
             parameters = [i for i in model_params if i in self.parameters]
-
-            #use default model parameters if user input parameters is empty, else
-            #   use user-specified parameters.
+            
+            #use default model parameters if user input parameters is empty {}, else
+            #use user-specified parameters
             if (parameters != {} or parameters != []):
                 model = PLSRegression(**self.parameters)
             else:
@@ -194,7 +212,8 @@ class Model():
             else:
                 model = Ridge()
 
-        elif (self.algorithm.lower().strip() == 'sgd'):
+        elif (self.algorithm.lower().strip() == 'sgd' or \
+            self.algorithm.lower().strip() == 'stochasticgradientdescent'):
 
             model_params = set(dir(SGDRegressor()))
             parameters = [i for i in model_params if i in self.parameters]
@@ -216,7 +235,8 @@ class Model():
             else:
                 model = GradientBoostingRegressor()
 
-        elif (self.algorithm.lower().strip() == 'svr'):
+        elif (self.algorithm.lower().strip() == 'svr' or \
+            self.algorithm.lower().strip() == 'supportvectorregression'):
 
             model_params = set(dir(SVR()))
             parameters = [i for i in model_params if i in self.parameters]
@@ -227,7 +247,8 @@ class Model():
                 model = SVR()
 
         elif (self.algorithm.lower().strip() == 'knn' or \
-           self.algorithm.lower().strip() == 'kneighborsregressor'):
+           self.algorithm.lower().strip() == 'kneighborsregressor' or \
+           self.algorithm.lower().strip() == 'knearestneighbors'):
 
             model_params = set(dir(KNeighborsRegressor()))
             parameters = [i for i in model_params if i in self.parameters]
@@ -236,13 +257,14 @@ class Model():
                 model = KNeighborsRegressor(**self.parameters)
             else:
                 model = KNeighborsRegressor()
+        #no matching valid algorithm/model found
         else:
-            raise ValueError('Input Algorithm ({}) not found in available valid models.'.
+            raise ValueError('Input Algorithm ({}) not found in available valid models: {}'.
                 format(self.algorithm, self.valid_models))
 
         return model
 
-    def train_test_split(self, X, Y, scale=True, test_size=0.2, random_state=None, shuffle=True):
+    def train_test_split(self, test_split=0.2, scale=True, random_state=None, shuffle=True):
         """
         Split the X and Y input features and labels into random train and test
         subsets. By default a 80:20 split will be used, whereby 80% of the data
@@ -252,53 +274,49 @@ class Model():
 
         Parameters
         ----------
-        :X : np.ndarray
-            array of feaure data.
-        :Y : np.ndarray
-            array of observed label values.
         :scale : bool (default=True)
             if true then scale the features such that they are standardised.
-        :test_size : float (default=0.2)
+        :test_split : float (default=0.2)
             proportion of the total dataset to use for testing.
         :random_state : float (default=None)
             Controls the shuffling applied to the data before applying the split.
-            Popular integer random seeds are 0 and 42.
+            Popular integer random seeds are 0 and 42, None by default.
         :shuffle : bool (default=True)
             Whether or not to shuffle the data before splitting.
 
         Returns
         -------
-        self.X_train, self.X_test, self.Y_train, self.Y_test : np.ndarray
+        :self.X_train, self.X_test, self.Y_train, self.Y_test : np.ndarray
             splitted training and test data features and labels.
         """
         #validate that X and Y arrays are of the same size
-        if (len(X) != len(Y)):
-            raise ValueError('X and Y input parameters must be of the same length, X: {}, Y: {}.'.
-                format(len(X), len(Y)))
+        if (len(self.X) != len(self.Y)):
+            raise ValueError('X and Y input parameters must be of the same length - X: {}, Y: {}.'.
+                format(len(self.X), len(self.Y)))
 
         #reshape input arrays to 2D arrays
-        if (X.ndim != 2):
-            X = np.reshape(X, (-1,1))
-        if (Y.ndim != 2):
-            if (isinstance(Y, pd.Series)):
-                Y = np.reshape(Y.values, (-1,1))
+        if (self.X.ndim != 2):
+            self.X = np.reshape(self.X, (-1,1))
+        if (self.Y.ndim != 2):
+            if (isinstance(self.Y, pd.Series)):
+                self.Y = np.reshape(self.Y.values, (-1,1))
             else:
-                Y = np.reshape(Y, (-1,1))
+                self.Y = np.reshape(self.Y, (-1,1))
 
         #if invalid test size input then set to default 0.2
-        if (test_size <= 0 or test_size >=1):
-            test_size = 0.2
+        if (test_split <= 0 or test_split >=1):
+            test_split = 0.2
 
-        #setting test_size attribute
-        self.test_split = test_size     
+        #setting test_split attribute
+        self.test_split = test_split     
 
         #scale training data X, if scale=True
         if (scale):
-            X = StandardScaler().fit_transform(X)
+            self.X = StandardScaler().fit_transform(self.X)
 
         #split X and Y into training and test data
-        X_train, X_test, Y_train, Y_test = train_test_split(X, Y,
-            test_size=test_size, random_state=random_state, shuffle=shuffle)
+        X_train, X_test, Y_train, Y_test = train_test_split(self.X, self.Y,
+            test_size=test_split, random_state=random_state, shuffle=shuffle)
 
         #set X and Y attributes
         self.X_train = X_train
@@ -339,7 +357,7 @@ class Model():
         """
         return self.model_fit.predict(self.X_test)
 
-    def save(self, save_folder):
+    def save(self, save_folder, model_name="model.pkl"):
         """
         Save fitted model to specified save_folder.
 
@@ -352,60 +370,74 @@ class Model():
         -------
         None
         """
-        save_path = os.path.join(save_folder, 'model.pkl')
-
+        #append pickle file extension if not present in filename
+        if (os.path.splitext(model_name)[1] == "" and 
+            os.path.splitext(model_name)[1] != "pkl"):
+            model_name = model_name + ".pkl"
+        
+        #set save path to folder + filename
+        save_path = os.path.join(save_folder, model_name)
+        
+        #save model in pickle format
         try:
             with open(save_path, 'wb') as file:
                 pickle.dump(self.model, file)
         except (IOError, OSError, pickle.PickleError, pickle.UnpicklingError):
             print("Error pickling model with path: {}.".format(save_path))
 
-    def hyperparameter_tuning(self, parameters={}, metric='r2', cv=5, n_jobs=None):
+    def hyperparameter_tuning(self, param_grid={}, metric='r2', cv=5, n_jobs=None, verbose=2):
         """
         Hyperparamter tuning of model to find its optimal arrangment of parameters
         using a Grid Search.
 
         Parameters
         ----------
-        :parameters : dict (default={})
-            dictionary of parameter names and their values.
+        :param_grid : dict (default={})
+            dictionary/grid of selected models' parameter and the potential values of each
+            that you want to tune.
         :metric : str (default=r2)
             scoring metric used to evaluate the performance of the cross-validated
-            model on the test set.
+            model on the test set, R2 by default. List of available scoring metrics
+            can be found in documentation:
+            https://scikit-learn.org/stable/modules/model_evaluation.html#scoring-parameter
         :cv : int (default=5)
             Determines the cross-validation splitting strategy.
         :n_jobs : int (default=None)
             Number of jobs to run in parallel. None means 1 job.
+        :verbose: int (default=2)
+            verbosity of output during tuning process. The values and what they mean 
+            for this parameter can be found on the documentation:
+            https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html
 
         Returns
         -------
         None
         """
-        #input 'parameters' parameter must be a dict, if not raise error
-        if not (isinstance(parameters, dict)):
-            raise TypeError('Parameters argument must be of type dict, not of type {}.'.format(type(parameters)))
+        #input 'param_grid' parameter must be a dict, if not raise error
+        if not (isinstance(param_grid, dict)):
+            raise TypeError('param_grid argument must be of type dict, got type {}.'.format(type(parameters)))
 
         #input metric must be in available scoring metrics, if not raise error
         if (metric not in sorted(SCORERS.keys())):
-            raise UndefinedMetricWarning('Invalid scoring metric, \
-                {} not in available Sklearn Scoring Metrics: {}.\n'.format(metric, SCORERS.keys()))
+            raise UndefinedMetricWarning('Invalid scoring metric, {} not in available Sklearn Scoring Metrics: {}.\n'\
+                .format(metric, SCORERS.keys()))
 
         #cv must be of type int and be between 5 and 10, if not then default of 5 is used
         if not ((isinstance(cv, int)) or (cv<5 or cv>10)):
             cv = 5
 
         #iterate through all parameter names to check if they are correct for model,
-        #  if parameter not found in model params then delete from dictionary
-        for p in list(parameters.keys()):
+        #if parameter not found in model params then delete from dictionary
+        for p in list(param_grid.keys()):
             if (p not in (list(self.model.get_params().keys()))):
-                del parameters[p]
+                del param_grid[p]
 
         #create deep copy of model
         model_copy = deepcopy(self.model)
 
         #grid search of hyperparameter space for model
-        grid_search = GridSearchCV(estimator=model_copy, param_grid=parameters, \
-            cv=cv, scoring=metric, n_jobs=n_jobs, verbose=2, error_score=0)
+        grid_search = GridSearchCV(estimator=model_copy, param_grid=param_grid, \
+            cv=cv, scoring=metric, n_jobs=n_jobs, verbose=verbose, error_score=0)
 
         #fit X and Y to best model found in grid search
         grid_result = grid_search.fit(self.X_train, self.Y_train)
@@ -417,12 +449,12 @@ class Model():
 
         #predict values of unseen test data using best found model
         best_model_pred = grid_result.predict(self.X_test)
-
+        
         #create instance of Evaluate class and calculate metrics from best model
         eval = Evaluate(self.Y_test,best_model_pred)
-
+        
         #print out results of grid search
-        print('\n#############################################################')
+        print('\n############################################################')
         print('################### Hyperparamter Results ###################')
         print('#############################################################\n')
 
@@ -430,18 +462,20 @@ class Model():
         print('# Best Params -> {}'.format(grid_result.best_params_))
         print('# Model Type -> {}'.format(repr(self)))
         print('# Scoring Metric -> {}'.format(metric))
-        print('# CV -> {}'.format(cv))
+        print('# Number of CV folds -> {}'.format(cv))
         print('# Test Split -> {}'.format(self.test_split))
 
-        print('########################## Metrics ###########################\n')
+        print('######################### Metrics ###########################\n')
         print('# Best Score (R2) -> {}'.format(grid_result.best_score_))
         print('# RMSE -> {} '.format(eval.rmse))
         print('# MSE -> {} '.format(eval.mse))
         print('# MAE -> {}'.format(eval.mae))
         print('# RPD -> {}'.format(eval.rpd))
         print('# Explained Variance -> {}\n'.format(eval.explained_var))
-        print('###############################################################')
+        print('##############################################################')
         
+        self.grid_result = grid_result
+
     def model_fitted(self):
         """
         Return if model has been fitted, true or false.
@@ -457,7 +491,7 @@ class Model():
         """
         return (self.model_fit != None)
 
-    def feature_selection(self, X, Y, method=""):
+    def feature_selection(self, method=""):
         """
         Feature selection/dimensionality reduction on dataset and models.
         Return the best applicable features found using the technique selected
@@ -465,10 +499,6 @@ class Model():
 
         Parameters
         ----------
-        :X : np.ndarray
-            training data.
-        :Y : np.ndarray
-            training data labels.
         :method : str (default="")
             feature selection method to use.
 
@@ -481,20 +511,50 @@ class Model():
         ----------
         [1] https://scikit-learn.org/stable/modules/feature_selection.html
         """
-        valid_feature_selection = []
+        #list of available sklearn feature selection techniques
+        valid_feature_selection = ["selectkbest", "chi2", "variancethreshold", "rfe", 
+            "selectfrommodel", "sequentialfeatureselector"]
 
         #get closest valid feature selection method
         feature_matches = get_close_matches(method.lower().strip(), [item.lower().strip() \
             for item in valid_feature_selection], cutoff=0.5)
 
         #apply feature selection method according to input parameter
-        if feature_matches == 'selectkbest':
-            X_new = SelectKBest(chi2, k=2).fit_transform(X, Y)
+        if (feature_matches == 'selectkbest'):
+            X_new = SelectKBest(chi2, k=1).fit_transform(self.X, self.Y)
+        elif (feature_matches == "variancethreshold"):
+            X_new = VarianceThreshold(1).fit_transform(self.X, self.Y)
+        elif (feature_matches == "chi2"):
+            X_new = chi2().fit_transform(self.X, self.Y)
+        elif (feature_matches == "rfe"):
+            pass
+        elif (feature_matches == "sequentialfeatureselector"):
+            pass
+        elif (feature_matches == "selectfrommodel"):
+            pass
+        else:
+            X_new = SelectKBest(chi2, k=2).fit_transform(self.X, self.Y)
 
         return X_new
         
 ######################          Getters & Setters          ######################
 
+    @property
+    def X(self):
+        return self._X
+
+    @X.setter
+    def X(self, val):
+        self._X = val
+
+    @property
+    def Y(self):
+        return self._Y
+
+    @Y.setter
+    def Y(self, val):
+        self._Y = val
+
     @property
     def model(self):
         return self._model
@@ -551,18 +611,18 @@ class Model():
     def valid_models(self,val):
         self._valid_models = val
 
-################################################################################
-
     def __str__(self):
-        return "Model of type {} using {} parameters, model fit = {}.".format(
+        return "Model of type {} using parameters {}, model has been fitted = {}.".format(
             type(self.model).__name__, self.parameters, self.model_fitted())
 
     def __repr__(self):
+        """ Object representation of class instance """
         return type(self.model).__name__
 
     def __eq__(self, other):
+        """ Checking if 2 sklearn models are the same """
         return self.model == other.model
 
     def __sizeof__(self):
         """ Get size of sklearn model """
-        return self.model.__sizeof__()
+        return self.model.__sizeof__()
\ No newline at end of file
